#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"





###############################################################################
#       OHJELMA:            SPRO
#       MODULI:             spro.tcl
#       TEKIJ:             Anssi Laakso
#       MUUTOSHISTORIA:     5.7.1999    changeweek (uusi funktio), jota 
#                                       kytetn ohjelmaa pivitettess;
#                                       listty Const-taulukon kautta 
#                                       kytettvi globaaleja muuttujia;
#                                       nytt pivitetn vlittmsti 
#                                       kun ohjelma kynnistetn
#
#
###############################################################################





# Globaalien muuttujien esittely  
global Const env x1 x2 y1 y2 weekyear
 
# Tehtvmerkintjen vrit ja frekvenssi
set Const(work_week_late) red
set Const(work_week_spare) lightgreen
set Const(work_week_estimate) white   
set Const(work_week_done) #008eff 
set Const(work_week_free) orange
set Const(work_week_premature) lightgray
set Const(work_week_transition) yellow
set Const(show_week_numbers) 1
set Const(week_frequence) 5

# Resurssimerkintjen vrit
set Const(resource_usage_20) #ffdaf4 
set Const(resource_usage_50) #ffd6de 
set Const(resource_usage_80) #ffaecb 
set Const(resource_usage_120) #ffa598 
set Const(resource_usage_MAX) red

# Tehtvlaatikoiden koot
set Const(task_small_box) 4 
set Const(task_big_box) 0

# Resurssilaatikoiden koot
set Const(resource_box_20) 5
set Const(resource_box_50) 4
set Const(resource_box_80) 3
set Const(resource_box_120) 2 
set Const(resource_box_MAX) 0 

# Ohjelman versionumero
set Const(Spro_version) 1.1

# Ohjelman kyttmt fontit
set Const(week_font)            {helvetica 12 normal}
set Const(name_font)            {helvetica 12 normal}
set Const(headline_font)        {times 18 normal}
set Const(task_halfway_font)    {times 18 bold} 

# Koordinaatisto-pisteiden alustus 
set x1 0
set x2 0
set y1 0
set y2 0

# Ikkunanmuuttaminen 
set Const(windowresizeable) 0

# Luetaan ohjelmatiedostot 
source $env(SPRO_BIN)/main.tcl
source $env(SPRO_BIN)/share.tcl
source $env(SPRO_BIN)/read.tcl

# Komentorivin parametrin lukeminen ja tarkistus
# Kutsutaan aliohjelmia:        dateformat_ok (share.tcl)
#                               add_century (share.tcl)
#                               getweekyear (share.tcl)
if { $argc == 0 } {                                    
    set t [clock seconds]
    set week [clock format $t -format %W]
    set non_valid_week [string index $week 0]       

    if {$non_valid_week==0} {
        set week [string index $week 1] 
        if { $week==0} {      
            set week 1
        }
   }
   set year [clock format $t -format %Y] 
   set weekyear [concat $week/$year]
} else  { 
    set weekyear [lindex $argv 0] 
    if {![dateformat_ok $weekyear]} {
        puts "Virheellinen pivys"
        exit
    }
    set weekyear [add_century $weekyear]
    getweekyear $weekyear week year
}
#------------------------------------------------------------------------------
# Funktio changeweek    
# Parametrit:               direction       -string 
#
# Funktio muuttaa tarkasteluviikkoa yhden viikon menneisyyteen tai 
# tulevaisuuteen riippuen siit kumpaa painiketta "<-" tai "->" on painettu. 
# Pivit-napin painaminen ei muuuta tarkasteluviikkoa.
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       addweek
#                           make_task_order
#                        
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Uusi funktio johtuen painikkeiden "<-" 
#                                       ja "->" lismisest kyttliittymn.   
#                               
#------------------------------------------------------------------------------
proc changeweek {direction} {

        global weekyear
        # painettu "->"-painiketta ..
        if { $direction=="+"} {
            set weekyear [addweek $weekyear 1]
        }
        # painettu "<-" -painiketta ..
        if { $direction=="-"} {
            set weekyear [addweek $weekyear -1]
        }

        make_task_order $weekyear
}

# Kyttliittymn alustus
wm title .  "Spro: <Ei projektia>"
frame       .buttons
button      .buttons.eteenpain -text "->" -command "changeweek +"
button      .buttons.taaksepain -text "<-" -command "changeweek -"
button      .buttons.toimi -text Pivit -command "changeweek none"
button      .buttons.lopeta -text Lopeta -command "destroy ." 
button      .buttons.info -text Infoa -command "print_info;.buttons.info \
            configure -state disabled"
pack        .buttons -side bottom -fill x 
pack        .buttons.lopeta -side right -padx 20 -pady 7
pack        .buttons.eteenpain -side right -padx 0 -pady 7 
pack        .buttons.toimi -side right -padx 0 -pady 7
pack        .buttons.taaksepain -side right -padx 0 -pady 7
pack        .buttons.info -side right -padx 20 -pady 7
frame       .raami
scrollbar   .raami.yvieritys -orient vertical -width 10 \
            -command ".raami.sproCanv yview"
canvas      .raami.sproCanv -width 500 -height 200 -background white\
            -scrollregion {0 0 100 100} \
            -xscrollcommand ".raami.xvieritys set" \
            -yscrollcommand ".raami.yvieritys set" 
scrollbar   .raami.xvieritys -orient horizontal -width 10 \
            -command ".raami.sproCanv xview"
pack        .raami -expand 1 -fill both -padx 1 -pady 1
grid        rowconfig .raami 0 -weight 1 -minsize 0
grid        columnconfig .raami 0 -weight 1 -minsize 0
grid        .raami.sproCanv -padx 1 -in .raami -pady 1 \
            -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
grid        .raami.xvieritys -in .raami -padx 1 -pady 1 \
            -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
grid        .raami.yvieritys -in .raami -padx 1 -pady 1 \
            -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
wm geometry . +200+200            
wm minsize . 300 100

# Ohjelma ajetaan heti kynnistyksen yhteydess 
make_task_order $weekyear    



