




###############################################################################
#       OHJELMA:            SPRO
#       MODULI:             main.tcl
#       TEKIJ:             Anssi Laakso
#       MUUTOSHISTORIA:     5.7.1999        addweek
#                                           check_cycles
#                                           draw_markers
#                                           make_to_currentweek
#                                           make_task_order 
#                                           draw_rectangle
#                                           make_to_end
#
###############################################################################





#------------------------------------------------------------------------------
# Funktio addweek    
# Parametrit:               weekyear        -string (esim. 12/1999)
#                           weeks           -integer (esim. 2)
#
# Funktio lis parametriin weekyear, weeks verran viikkoja. Funktio tarkistaa
# mys mahdollisen vuoden vaihtumisen. TM vain jos viikkoja kasvatetaan
# maltillinen mr (1 viikko kerrallaan) !!
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       getweekyear (share.tcl)
#                        
# Palautusparametrit:       weekyear        -string    (esim 14/1999)
#
# Historia:                 5.7.1999    Korjattu funktiota siten, ett se
#                                       toimii mys silloin kun viikkojen
#                                       summaksi saadaan 0 
#                                           
#------------------------------------------------------------------------------
proc addweek {weekyear weeks} {

    getweekyear $weekyear weeky yeary
    set weeky [expr $weeky + $weeks]

    # jos viikkojen yhteenlaskettu mr on 53 , vuosi ei vaihdu
    if {$weeky==53||$weeky==0} {
        set a 0
        if { $weeky==0} {
            set weeky 53
            incr yeary -1
        }
    } else  {
        set a [expr $weeky / 53]    
    }

    # jos vuosi vaihtuu niin lasketaan uusi viikko/vuosi
    if {$a} {
        set b [expr $weeky % 53]
        set weeky $b
        set yeary [expr $a + $yeary]
    }

    set weekyear [concat $weeky/$yeary]
    return $weekyear
}
#-------------------------------------------------------------------------------
# Funktio make_end_dates    
# Parametrit:               taskprop        -taulukko
#                           tasks           -lista
#
# Funktio luo tehtville tasks alustavat loppumisajankohdat taulukosta 
# taskprop lytyvien tietojen perusteella.
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       getweekyear (share.tcl)
#                        
# Palautusparametrit:       -
#
# Historia:
#-------------------------------------------------------------------------------
proc make_end_dates {taskprop tasks} {

    upvar $taskprop taskArray

    # kullekin tehtvlle lasketaan loppumisajankohta
    foreach task $tasks {
        getweekyear $taskArray($task,start) startingweek startingyear
        set enddate [expr $taskArray($task,duration)+$startingweek-1] 
        set a [expr $enddate / 53]
        if { $a==53} {
            set a 0
        }
        set endyear $startingyear
        if {$a} {    
            set b [expr $enddate % 53]      
            set enddate $b
            set endyear [expr $a+$startingyear]
        }
        set enddate [concat $enddate/$endyear]
        set taskArray($task,end) $enddate   
    }
}
#------------------------------------------------------------------------------
# Funktio get_startdate    
# Parametrit:               taskprop        -taulukko
#                           tasks           -lista
#
# Funktio hakee taulukosta taskprop tehtvist tasks ensimmisen aloitusviikon. 
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       aolderb    
# 
# Palautusparametrit:       startdate       -string
#
# Historia:
#------------------------------------------------------------------------------
proc get_startdate {taskprop tasks} {

    upvar $taskprop taskArray    
    set startdate [concat 100/3000]

    # haetaan tehtvien aloitushetkist ensimmiseksi alkavaa
    foreach task $tasks {
        if {[aolderb $taskArray($task,start) $startdate]} {
            set startdate $taskArray($task,start)
        }
    }
    return $startdate
}

#------------------------------------------------------------------------------
# Funktio dfs    
# Parametrit:               dfsTask         -string
#                           taskArray       -taulukko
#
# Globaalit muuttujat:      chain           -lista
#                           cycle           -integer (boolean)
#                
# Suoritetaan syvyyshaku alkaen "solmusta" dfsTask. Edetn taulukon taskArrayn
# tietojen perusteella mahdollista sykli. Jos sykli lydetn, lopetetaan 
# etsint. Listaan chain talletetaan kuljettu polku, ja muuttujaan cycle 
# ilmaisee jos sykli on lydetty.
# 
# Ulkoiset vaikutukset:     -
#
# 
# Kutsuu aliohjelmia:       dfs (rekursio)
# 
# Palautusparametrit:       "return"
#
# Historia:
#------------------------------------------------------------------------------
proc dfs {dfsTask taskArray} {

    upvar $taskArray dfsArray
    global chain
    global cycle

    # jos sykli on lydetty, voidaan syklin etsint lopettaa
    if {$cycle} {
        return
    }
    # listn nykyinen tehtv kuljettuun polkuun
    lappend chain $dfsTask

    # jos nykyisell tehtvll on edeltji ..
    if {$dfsArray($dfsTask,predecessors)!={}} {
        # ja nykyist tehtv ei ole jo vierailtu ..
        if {$dfsArray($dfsTask,visited)!=2} {
            # merkitn se vierailluksi
            set dfsArray($dfsTask,visited) 1
        } 
    }
    # kullekkin nykyisen tehtvn edeltjlle ..
    foreach predTask $dfsArray($dfsTask,predecessors) {

        # jos EDELTJtehtvll on edeltji ..
        if {$dfsArray($predTask,predecessors)!={}} {

            # ja edeltjtehtvss on vierailtu aikaisemmin ..
            if { $dfsArray($predTask,visited)!=0 } {
                # lydettiin sykli!
                set dfsArray($predTask,clash) $predTask    
                set cycle 1
                return 

            # ja edeltjtehtvss ei vierailtu -> dfs edeltjtehtv ..
            } else  {
                dfs $predTask dfsArray                       
            }

        # jos EDELTJtehtvll ei ole edeltji ..
        } else  {
            # poistetaan kuljettu polku TAULUKOSTA 
            foreach path_elem $chain {
                 if {$dfsArray($path_elem,visited)!=2} {
                     set dfsArray($path_elem,visited) 0
                 } 
            }
            # asetetaan polkulista tyhjksi
            set chain {}
        }
   }
}
#------------------------------------------------------------------------------
# Funktio check_cycles    
# Parametrit:               taskprop        -taulukko
#                           tasks           -lista
#
# Funktio tarkistaa kunkin listan tasks tehtvn osalta syklit. Taulukko 
# taskprop kopioidaan aputaulukkooon, jota kytetn etsinnss hyvksi. 
# Etsint lopetetaan kun sykli lydetn.
#
# Ulkoiset vaikutukset: Tulostaa viesti-ikkunan, jos sykli lytyy. Luo
#                       tilapistiedoston tmp.txt , jota grep
#                       kytt.
#
# 
# Kutsuu aliohjelmia:       dfs
#                           error_message (share.tcl)
#
# Ulkoisten ohjelmien kytt:    grep
# 
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Muutettu ympristmuuttujan nimi
#------------------------------------------------------------------------------
proc check_cycles {taskprop tasks} {

    upvar $taskprop taskArray    
    global chain cycle env
    set chain {}
    set cycle 0

    foreach task $tasks {

        # jos sykli lydetty, ei etsint jatketa
        if {$cycle} {
            break
        }
 
        # alustetaan dfsArray
        foreach task2 $tasks {
            set dfsArray($task2,visited) 0
            set dfsArray($task2,predecessors) $taskArray($task2,forerunner)
        }

        # jos tehtvll edeltji, aloitetaan syvyyshaku
        if {$dfsArray($task,predecessors)!={}} {
            set dfsArray($task,visited) 2
            dfs $task dfsArray
            set dfsArray($task,visited) 0
        }
    }

    # jos sykli lydetty, kalastetaan syklin aiheuttanut tehtv
    set testi {}
    if { $cycle} {
        foreach task $tasks {
            catch [set testi $dfsArray($task,clash)]
        }
    }
    # tulostetaan virheilmoitus syklist, jos syklin aiheuttanut tehtv
    # lydettin
    if {$testi!={}} {
        set b [exec grep Edeltajat: $env(SPRO_PROJ)/perust.txt -n]   
        set tmp_file [open $env(SPRO_PROJ)/tmp.txt w+]
        puts $tmp_file $b                            
        close $tmp_file
        set b [exec grep $testi $env(SPRO_PROJ)/tmp.txt ]            
        error_message none $b 18
    }
}
#------------------------------------------------------------------------------
# Funktio aolderb    
# Parametrit:               a       -string
#                           b       -string
# 
# Palauttaa totuusarvon riippuen siit, onko a aikaisempi tai myhisempi
# ajankohta kuin b
# 
# Kutsuu aliohjelmia:       getweekyear (share.tcl)
# 
# Palautusparametrit:       totuusarvo
#
# HUOM:    palauttaa totuusarvon TOSI mys silloin jos a ja b ovat samat
#
# Historia:
#------------------------------------------------------------------------------
proc aolderb {a b} {

    getweekyear $a aweek ayear
    getweekyear $b bweek byear

    # palautetaan tosi jos a aikaisempi tai SAMA(!!) kuin B
    if {$ayear<$byear} {
        return 1
    } else {
        if {$ayear==$byear} {
            if {$aweek<=$bweek} {
                return 1 
            }
        }
    }
    return 0 
}
#------------------------------------------------------------------------------
# Funktio draw_rectangle    
# Parametrit:               task            -string
#                           shape           -string
#                           curr_week       -integer (1-53)
#                           curr_year       -integer (1990-2089)
#                           size            -integer 
#
# Globaalit muuttujat:      x1              -integer
#                           x2              -integer
#                           y1              -integer
#                           y2              -integer
#                           Const           -taulukko
#
# Piirretn kanvaasiin .raami.sproCanv koordinaatteihin x1,x2,y1,y2 
# erikokoinen/erivrinen neli riippuen parametreista shape, size, Const. 
# Piirretn kanvaasiin .raami.sproCanv koordinaatteihin (x1, y1) viikko 
# curr_week. Kiinnitetn tehtvneliihin tgi, jos se on "normaalikokoinen".
#  
# Ulkoiset vaikutukset:     Tulostus kanvaasiin .raami.sproCanv
#
# 
# Kutsuu aliohjelmia:       -
# 
# Palautusparametrit:       -
# HUOM: Jos tehtvneli on normaalikokoa, ei siis rokulia tms., kiinnitetn
# siihen tgi jota kytetn hyvksi resurssiasteita laskettaessa!!
#
# Historia:                 5.7.1999    Muutettu funktiota siten, ett 
#                                       tehtv neliiden koot otetaan 
#                                       glob.muuttujasta.
#------------------------------------------------------------------------------
proc draw_rectangle {task shape curr_week curr_year size} {

    global x1 x2 y1 y2 Const
    set small_rect 0

    if {$task==$shape} { 
        set color $shape 
        incr small_rect
    } else { 
        #asetetaan neliiden vri ja koko globaalin muuttujan perusteella
        switch $shape {
            "lgreen" { 
                set color $Const(work_week_spare)
                set size $Const(task_small_box)
                incr small_rect 
            }
            "dgreen" { 
                set color $Const(work_week_estimate)
                set size $Const(task_big_box)
            }
            "yellow" { 
                set color $Const(work_week_transition)
                set size $Const(task_small_box)
                incr small_rect 
                }
            "blue" { 
                set color $Const(work_week_done)
                set size $Const(task_big_box)
            }
            "red" { 
                set color $Const(work_week_late)
                set size $Const(task_big_box)
            }
            "orange" { 
                set color $Const(work_week_free)
                set size $Const(task_small_box)
                incr small_rect 
            }
            "white" { 
                set color $Const(work_week_premature)
                set size $Const(task_small_box)
                incr small_rect
            }
       }
    }

    if {$small_rect} {  
         # pienikokoinen tehtv/resurssi-neli
        .raami.sproCanv create rect $x1 [expr $y1+$size] $x2 [expr $y2-$size] \
        -fill $color
    } else {
        # normaalikokoinen tehtvneli, jolloin siihen
        # liitetn tgi. Tt kytetn laskettaessa resurssien kyttasteita!     
        .raami.sproCanv create rect $x1 [expr $y1+$size] $x2 [expr $y2-$size] -fill $color \
        -tag d$curr_week/$curr_year 
        .raami.sproCanv addtag $task withtag d$curr_week/$curr_year
     }

    # viikkonumeroinnin tiheys  
    if {$Const(show_week_numbers)} {
        if {[expr $curr_week % $Const(week_frequence) ]==0} {
            .raami.sproCanv  create text [expr $x1+10] [expr $y1+10] \
            -text "$curr_week" -anchor center -tags text -font $Const(week_font)
        }
    }        
}
#------------------------------------------------------------------------------
# Funktio process_span    
# Parametrit:               task            -string
#                           start           -string
#                           end             -string
#                           color           -string
#
# Globaalit muuttujat:      x1              -integer
#                           x2              -integer
#                           y1              -integer
#                           y2              -integer
#
# Funktio ky lpi tehtvn task annetun aikajnteen vlilt start-end. 
#
# Ulkoiset vaikutukset:     -   
# 
# Kutsuu aliohjelmia:       draw_rectangle
#                           addweek
#                           aolderb     
# 
# Palautusparametrit:       -
#
# Historia:
#------------------------------------------------------------------------------
proc process_span {task start end color} {

    global x1 x2 y1 y2
    set size 0

    # jos tehtv on liukumaa, siirtym, rokulia, asetetaan sen koko
    # pienemmksi kuin normaalisti
    if {$color=="lgreen" || $color=="orange" ||$color=="yellow"} {
        set size 4
    }

    # kydn lpi aikajnne start -> end, ja tulostetetaan nelit.
    set end [addweek $end -1]     
    while {[aolderb $start $end]} {
        set x2 [expr $x1+20]
        set y2 [expr $y1+20]
        if {$color!=0} {
            getweekyear $start curr_week curr_year
             draw_rectangle $task $color $curr_week $curr_year 0
        }
        incr x1 20
        set start [addweek $start 1]    
    }
}
#------------------------------------------------------------------------------
# Funktio get_task_end    
# Parametrit:               taskprop        -taulukko
#                           tasks           -lista
#
# Funktio hakee tehtvist tasks taulukosta taskprop lytyv viimeisint 
# ksiteltv viikkoa.
# 
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       aolderb
# 
# Palautusparametrit:       max_week        -string
#
# Historia:
#------------------------------------------------------------------------------
proc get_task_end {taskprop tasks} {

    upvar $taskprop taskArray
    set max_week 00/00

    # haetaan tehtvien joukosta viimeisimmksi loppuvaa tehtv
    foreach task $tasks {
        if {[aolderb $max_week $taskArray($task,end)]}  {
            set max_week $taskArray($task,end)
        }
    }

    return $max_week
}
#------------------------------------------------------------------------------
# Funktio  draw_markers    
# Parametrit:               resources       -lista
#                           tasks           -lista
#                           taskprop        -taulukko
#                           weekyear        -string
#
# Piirretn kanvaasiin ksiteltvn ajankohdan sek proj.loppu ajankohta
# pystyakseleina.
#
# Ulkoiset vaikutukset:     Tulostaa kanvaasiin .raami.sproCanv pystyakselit,
#                           vuodet, viikot, erotinviivan.
#
# Kutsuu aliohjelmia:       get_task_end
#                           aolderb
#                           getweekyear (share.tcl) 
# 
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Muutettu piirrettvien pystyakselien
#                                       piirtokohtaa siten, ett jana alakohta 
#                                       alkaa selvtsti viimeisen resurssin 
#                                       alapuolelta.
#------------------------------------------------------------------------------
proc draw_markers {resources tasks taskprop weekyear startdate} {
 
    global x1 y1 Const
    upvar $taskprop taskArray
    set enddate [get_task_end taskArray $tasks]
    set project_end $taskArray(project_end) 

    # mritelln viimeisimmksi ksiteltvn viikon hetki.
    if {[aolderb $project_end $weekyear]} {
        set last_week $weekyear
    } else  {
        set last_week $project_end
    }
    if {[aolderb $last_week $enddate]} {
        set last_week $enddate
    }

    # mritelln kanvaasiin piirrettvien janojen koordinaatteja
    set yy2 [llength $resources] 
    set yy3 [llength $tasks]
    set yy2 [expr $yy2+$yy3]
    set yy3 [expr $yy3*30]
    incr yy3 70 
    set yy2 [expr $yy2*45]
    set first_week 1
    set y2 $yy2
 
    set apu $startdate   
    set x1 110
    set x2 120
    getweekyear $apu viikko vuosi
    .raami.sproCanv create text $x2 10 -text $vuosi -anchor center -font $Const(week_font)

    # ksitelln aikavli apu -> last_week. Tulostetaan vuosi joka
    # tasakymmenviikon kohdalle sek tarkasteluviikko ja projekin
    # lopetusviikko
    while {[aolderb $apu $last_week]} {
        getweekyear $apu viikko vuosi

        # viikon tulostus
        .raami.sproCanv create text $x2 22 -text $viikko -anchor center \
            -font $Const(week_font)

        if {$viikko==1 || $viikko==10 || $viikko==20 || $viikko==30 || \
            $viikko==40 || $viikko==52} {
            # vuoden tulostus 
            .raami.sproCanv create text $x2 10 -text $vuosi -anchor \
                center -font $Const(week_font)     
        }
        # tarkasteluviikon pystyakseli
        if {$apu==$weekyear} {
            .raami.sproCanv create line $x1 27 $x1 $yy2 -fill blue -width 3 \
            -arrow first
        }
        # projektin loppuviikon pystyakseli
        if {$apu==$project_end} {
            .raami.sproCanv create line $x1 27 $x1 $yy2 -fill black -width 2 \
            -arrow first
            }
        incr x1 20
        incr x2 20
        set apu [addweek $apu 1]
    }

    # tehtv- ja resurssitietojen erotusjana
    .raami.sproCanv create line 100 $yy3 $x2 $yy3 -fill grey -width 3 

}
#------------------------------------------------------------------------------
# Funktio make_resources  
# Parametrit:               resourceprop        -taulukko
#                           taskprop            -taulukko
#                           resources           -lista
#                           tasks               -lista
#                           weekyear            -string
#                           startdate           -string
#
# Ksitelln resurssit. Viikon mrytyminen saadaan kanvaasiin liitetyist 
# tgeist
#
# Ulkoiset vaikutukset:     Tulostaa kanvaasiin .raami.sproCanv resurssitiedot
#
# Kutsuu aliohjelmia:       get_task_end
#                           aolderb
# 
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Korjattu resurssilaskennan nyttmt
#                                       resurssiaste-% inforuutua
#                                       vastaaviksi
#------------------------------------------------------------------------------
proc make_resources {resourceprop taskprop resources tasks weekyear startdate} {

    global x1 x2 y1 y2 Const 
    upvar $resourceprop resourceArray
    upvar $taskprop taskArray
    set enddate [get_task_end taskArray $tasks]
    set project_end $taskArray(project_end)

    set x1 10

    # Resurssit-otsikon tulostus
    .raami.sproCanv create text $x1 [expr $y1+10] -text "Resurssit" \
    -anchor w -font $Const(headline_font)    
     incr y1 30

    set y1_alkup $y1

    # tulostetaan kunkin resurssin nimi omalle rivilleen
    foreach resource $resources {
       .raami.sproCanv create text $x1 [expr $y1+10] -text "$resource" \
        -anchor w  -font $Const(name_font)
        incr y1 30
    }

    incr x1 100
    set apu $startdate

    # ksitelln projektin ensimmisen ja viimeisen viikon vli. Kultakin
    # viikolta etsitn kanvaasin .raami.sproCanv tgit. Jos tgej lytyy,
    # saadaan niden kautta selville tehtvt, joita on "tehty". Niden
    # tehtvien resurssit kydn lpi ja lasketaan ksiteltvn resurssin
    # kyttaste 
    while {[aolderb $apu $enddate]} {
            set tasks [.raami.sproCanv gettags d$apu]
            set fetched_item [lsearch -exact $tasks d$apu]
            if { $fetched_item>-1} { 
                set tasks [lreplace $tasks $fetched_item $fetched_item]
            }
            if {$tasks==""} {
                set apu [addweek $apu 1]
                incr x1 20
                continue
            }
            set y1 $y1_alkup
            foreach resource $resources {
                  set default_amount $resourceArray($resource,quantity)
                  set fetched_item [lsearch $resourceArray($resource,exception) $apu*]

                if {$fetched_item!=-1} {
                    set r_exception [lindex $resourceArray($resource,exception) $fetched_item]
                    set r_exception [split $r_exception "|"]
                    set r_exception [lindex $r_exception 1]
                    set d_amount $r_exception
                } else  {
                    set d_amount $default_amount
                }

                set total_amount 0 
                foreach task $tasks {
                    set fetched_item [lsearch $taskArray($task,resources) $resource|*]
                    if {$fetched_item!=-1} {
                        set res_amount [lindex $taskArray($task,resources) $fetched_item] 
                        set res_amount [split $res_amount "|"]
                        set res_amount [lindex $res_amount 1]
                        incr total_amount $res_amount
                    }     
                }
                set percentage [expr $total_amount*100/$d_amount]
                
                while {1} {

                    if {$percentage<=20} {
                        set r_color $Const(resource_usage_20)
                        set size $Const(resource_box_20)
                        break
                    }
                    if {$percentage<=50} {
                        set r_color $Const(resource_usage_50)
                        set size $Const(resource_box_50)
                        break
                    }
                    if {$percentage<=80} {
                        set r_color $Const(resource_usage_80)            
                        set size $Const(resource_box_80)
                        break
                    }        
                    if {$percentage<=120} {
                        set r_color $Const(resource_usage_120)            
                        set size $Const(resource_box_120)
                        break
                    }
                    if {$percentage>120} {
                        set r_color $Const(resource_usage_MAX)
                        set size $Const(resource_box_MAX)
                        break
                    }
                    set r_color 0
                    break
                    
                  }
                # viikon nro. talteen    
                set temp [split $apu "/"]
                set temp [lindex $temp 0]
                set x2 [expr $x1+20]
                set y2 [expr $y1+20]
                # tulostetaan ko. viikko
                if {$r_color!=0} {
                    draw_rectangle $r_color $r_color $temp 1 $size
                }
                incr y1 30
            }
     set apu [addweek $apu 1]
     incr x1 20
    }
}
#------------------------------------------------------------------------------
# Funktio handle_rear    
# Parametrit:               taskprop        -taulukko
#                           tasks           -lista
#                           week            -string
#
# Ksitelln kaikkien tehtvien mahdolliset liukumat taaksepin.
#
# Ulkoiset vaikutukset:     Tulostaa kanvaasiin .raami.sproCanv liukumat
#                           taaksepin
#
# Kutsuu aliohjelmia:       aolderb
#                           getweekyear (share.tcl)
# 
# Palautusparametrit:       -
#
# Historia:
#------------------------------------------------------------------------------
proc handle_rear {taskprop tasks week} {

    global x1 x2 y1 y2
    upvar $taskprop taskArray

    # kydn lpi kaikki tehtvt
    foreach curr_task $tasks {
        if { $taskArray($curr_task,x1)==0} {
            # jos koord x1 on 0, on tehtv pttynyt ja sille ei tarvitse
            # piirt liukumaa taaksepin
            continue
        }
        set min $taskArray(project_end)
        set predecessors {} 
        lappend predecessors $taskArray(project_end)
        # samoin kuin kaikki mahdolliset tehtvt joilla on seuraaja
        foreach prede_task $tasks {
            if {$prede_task!=$curr_task} {
                # jos lytyy edeltji, liitetaan niiden alkuhetki listaan
                # predecessors
                if {[lsearch -exact $taskArray($prede_task,forerunner) $curr_task]!=-1} {
                    lappend predecessors $taskArray($prede_task,start)
                }
            } else {
                continue
            }
        }

       # haetaan aikaisinta ajankohtaa
        foreach pre $predecessors { 
            if {[aolderb $pre $min ]} {
                set min $pre
            }
        }

        # jos aikaisin ajankohta on ennen/sama kuin tarkasteluviikko, voidaan
        # jatkaa kierrosta. Ts. tehtvll ei ole liukumaa taaksepin.
        if {[aolderb $min $week]} {
            continue
        }

        # haetaan nykyisen tehtvn viimeiseksi piirretyn nelin koordinaatit
        set a1 $taskArray($curr_task,x1)
        set b1 $taskArray($curr_task,y1)

        # jos tarkasteluviikko aikaisemmin kuin tehtvn loppu, voidaan esitt
        # liukuma taaksepin
        set date1 [addweek $taskArray($curr_task,end) 1]
        set min [addweek $min -1]
        if {[aolderb $week $date1]} {

            # ksitelln aikajnne nykyisen tehtvn loppu -> seuraajan
            # alkuhetki/projektin loppu
            while {[aolderb $date1 $min]} {
                set x1 $a1
                set y1 $b1
                set x2 [expr $a1+20]
                set y2 [expr $b1+20]
                getweekyear $date1 viikko vuosi
                 draw_rectangle $curr_task lgreen $viikko 100 0
                incr a1 20    
                set date1 [addweek $date1 1]
            }
        }
    }
}
#---------------------------------------------------------------------------
# Funktio make_to_taskstart    
# Parametrit:               taskprop        -taulukko
#                           task            -string
#                           week            -string 
#                           task_Start      -string
#                           task_End        -string
#                           true_start      -string
#                           prev_End        -string
#                           startdate       -string
#                           future          -integer (boolean)
#
# Ksitelln viikot alkaen ensimmisest ksiteltvst viikosta tehtvn
# alkuun saakka. 
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       addweek
#                           process_span
#                           aolderb 
# 
# Palautusparametrit:       -
#
# Historia:
#---------------------------------------------------------------------------
proc make_to_taskstart {taskprop task week task_Start task_End true_start \
    prev_End startdate future} {

    upvar $taskprop taskArray
    upvar $task_Start task_start        
    upvar $task_End task_end
    upvar $prev_End prev_end

    if {$future} {
        set prev_end [addweek $prev_end 1] 
        if {$true_start == "-1/-1"} {   
            # ei edeltji ..
            if {$startdate!=$week} {
                # aloituskohdan etsiminen ekasta viikosta tarkasteluviikkoon    
                process_span $task $startdate $week 0 
             }
            # liukumaviikkojen esittminen tarkasteluviikosta tehtvn alkuun
            process_span $task $week $task_start lgreen
        } else {                        
            # on edeltji ..
            if {[aolderb $prev_end $task_start]} {

                # aloituskohdan etsiminen ekasta viikosta edellisen teht. loppuun
                process_span $task $startdate $prev_end 0

                if {[aolderb $prev_end $week]} {
                    # aloituskohdan etsiminen edell.tehtvn lopusta
                    # tarkasteluviikkoon

                    process_span $task $prev_end $week 0 
                    # liukumaviikkojen esittminen tarkasteluviikosta tehtvn
                    # alkuun
                    process_span $task $week $task_start lgreen


                } else  {
                    # liukumaviikkojen esittminen edellisen tehtvn lopusta 
                    # tehtvn alkuun saakka
                    process_span $task $prev_end $task_start lgreen
                }

                set task_end $taskArray($task,end)
                set $taskArray($task,start) $prev_end

            } else { 
                # aloituskohdan etsiminen ekasta viikosta tehtvn alkuun saakka
                process_span $task $startdate $task_start 0 

                # siirtymviikkojen esittmien tehtvn alkuperisest viikosta
                # tehtv todelliseen alkuviikkoon
                process_span $task $task_start $true_start yellow
                set $taskArray($task,start) $prev_end

                # koska siirtym on syntynyt, on tehtvn loppuviikkoa muutettava
                # vastaaavasti
                set start_yellow_week $task_start
                set end_yellow_week [addweek $true_start -1]
                set transition_weeks 0
                while {[aolderb $start_yellow_week $end_yellow_week]}  {
                    set start_yellow_week [addweek $start_yellow_week 1]
                    incr transition_weeks 
                }

                set taskArray($task,end) [addweek $taskArray($task,end) \
                    $transition_weeks]
                set task_end $taskArray($task,end)
                set task_start $true_start 
            } 
        } 
    } else  { 
        # aloituskohdan etsiminen ekan tehtvn alusta nykyisen tehtvn alkuun
        process_span $task $startdate $task_start 0 
    }    
}
#---------------------------------------------------------------------------
# Funktio make_to_currentweek    
# Parametrit:               taskprop            -taulukko
#                           task                -string 
#                           task_start          -string
#                           week                -string
#                           prev_end            -string
#                           completed_Weeks     -string 
#                           Early               -integer (boolean)
#
# Ksitelln viikot tehtvn alusta tarkasteluviikon alkuun saakka. 
#
# Ulkoiset vaikutukset:     Tulostaa kanvaasiin .raami.sproCanv tehtvn nimen,
#                           jos tehtv on puolivliss
#
# Kutsuu aliohjelmia:       addweek
#                           aolderb
#                           getweekyear (share.tcl)
#                           draw_rectangle
#
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Listty tehtvn pttymisen osoittava
#                                       merkint (nimen vritys). 
#                                       Korjattu virhe, joka johtui
#                                       esittelemttmst muuttujasta
#---------------------------------------------------------------------------
proc make_to_currentweek {taskprop task task_start week prev_end completed_Weeks \
    Early Orig_end} {

    upvar $taskprop taskArray
    upvar $completed_Weeks completed_weeks
    upvar $Early early
    upvar $Orig_end orig_end

    global x1 x2 y1 y2 Const

    set task_early 1
    set task_rip 0
    set duration $taskArray($task,new_duration)
    if { $duration==""} {
        set duration $taskArray($task,duration)
    } 

    # tarkistetaan tehtvn loppumerkin lytyminen ..
    set work_weeks $taskArray($task,work_weeks)
    set fetched_item -1
    set fetched_item [lsearch -exact $work_weeks X]
    if {$fetched_item!=-1} {
        set task_rip 1
        set work_weeks [lreplace $work_weeks $fetched_item $fetched_item]
        foreach rip_week $work_weeks {
            # jos tehdyiss viikoissa on viikkoja menneisyydess, on nm
            # poistettava listasta.
            if {[aolderb $rip_week $taskArray($task,start)] && $rip_week!=$taskArray($task,start)} {
                set fetched_item [lsearch -exact $work_weeks $rip_week]
                set work_weeks [lreplace $work_weeks $fetched_item $fetched_item]
            }
        }
    }

    set $taskArray($task,start) $task_start
    set date1 $task_start
    set date2 [addweek $week -1]
    # nyt ksitelln viikot tarkasteluviikkoon saakka 
    set lock 1 
    while {[aolderb $date1 $date2]} {
         set x2 [expr $x1+20]
         set y2 [expr $y1+20]
         getweekyear $date1 week1 year2

        # jos tehtvn alku aikaisempi kuin edellisen loppu, on esitettv
        # siirtym siihen saakka
        if {[aolderb $date1 $prev_end]} {
             draw_rectangle $task yellow $week1 $year2 0
            set taskArray($task,end) [addweek $taskArray($task,end) 1]     
            incr x1 20
            set date1 [addweek $date1 1]
            continue
        }

        # tarkistetaan tehdyt viikot ..
        set fetched_item [lsearch -exact $work_weeks $date1]
        if {$fetched_item!=-1} {
            incr completed_weeks 1       
            # tehtyjen viikkojen tarkistaminen: onko ylity-viikkoa 
            if {$duration<$completed_weeks} {
                # tehty ylity-viikko 
                draw_rectangle $task red $week1 $year2 0
                set taskArray($task,end) [addweek $taskArray($task,end) 1] 
            } else  {
                # tehty normaali viikko         
                draw_rectangle $task blue $week1 $year2 0
                set $taskArray($task,end) $date1  
            }
            incr x1 20         

            # poistetaan ksitelty tyviikko listasta
            set work_weeks [lreplace $work_weeks $fetched_item $fetched_item]

            # viimeisen ksitellyn viikon jlkeen tiedetn tehtvn todellinen     
            # lopetus viikko, jos tehtvll oli lopetuskoodi X. Orig_end ilmaisee
            # tehtvn todellisen lopetusviikon. Muuttujalla lock estetn
            # ylimriset orig_end muutokset.
            if {$task_rip && $work_weeks=="" && $lock} {
                if { $task_rip} {
                    .raami.sproCanv itemconfigure T$task -fill grey \
                    -font $Const(headline_font)
                    set early 1
                }
                set lock 0
                set orig_end $date1
            }
            set date1 [addweek $date1 1]

         } else { 
            # viikkoa ei oltu tehty ..
            if {$task_rip && $work_weeks==""} { 
                # tehtv pttynyt aikaisemmin
                if {$duration>$completed_weeks} {
                    # korjattava tehtvn loppuminen todellisuutta vastaavaksi  
                    set early_end [addweek $taskArray($task,end) \
                        [expr $duration-$taskArray($task,duration)]]
                    if {[aolderb $date1 $early_end]} {
                         draw_rectangle $task white $week1 $year2 0
                    }
                    if {$task_early} {
                        set task_early 0
                        # muuttuja asetetaan tll todeksi, kun tiedetn ett
                        # tehtv todellakin pttyi aikaisemmin
                        set early 1
                        .raami.sproCanv itemconfigure T$task -fill grey \
                        -font $Const(headline_font)
                    }
                }
            } else { 
                # viikko oli rokuliviikko
                draw_rectangle $task orange $week1 $year2 0
                set taskArray($task,end) [addweek $taskArray($task,end) 1]     
            }
            incr x1 20   
            set date1 [addweek $date1 1]
        }
    }

    # esitetn tehtvn puolivli, jos tehdyt viikot ovat puolivliss.
    set halfway [expr $duration / 2]
    if {![expr $duration % 2]==0} {
        incr halfway
    } 
    if {$completed_weeks!=0  && $completed_weeks==$halfway && !$early} {
        .raami.sproCanv itemconfigure T$task -fill blue\
        -font $Const(task_halfway_font)     
    }

}
#---------------------------------------------------------------------------
# Funktio make_to_end    
# Parametrit:               taskprop            -taulukko
#                           task                -string
#                           task_start          -string
#                           week                -string
#                           completed_Weeks     -integer
#                           early               -interger (boolean)
#                           future              -integer 
#
# Ksitelln viikot tarkisteluviikosta  alkaen tehtvn loppuun saakka. 
#
# Ulkoiset vaikutukset:     -   
#
# Kutsuu aliohjelmia:       addweek
#                           aolderb
#                           getweekyear (share.tcl)
#                           draw_rectangle
#
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Korjattu tilanne, jossa 
#                                       ylim. tulevaisuuteen osuvat 
#                                       rokulit pivittyivt vrin; 
#                                       mys ylim. liukumat taaksepin
#                                       korjattu
#                                           
#---------------------------------------------------------------------------
proc make_to_end {taskprop task task_start week completed_Weeks early future orig_end} {


    upvar $taskprop taskArray
    upvar $completed_Weeks completed_weeks
    global x1 x2 y1 y2
    set date1 $week

    # jos tehtvn aloitus jlkeen tarkasteluviikon, aloitetaankin tehtvn
    # alusta tarkasteluviikon sijaan 
    if {$future} {
        set date1 $task_start
    }
    # tarkistetaan ett uusi tehtvn kestoarvio otetaan huomioon
    set new_duration $taskArray($task,new_duration)  
    if {$new_duration==""} {
        set date2 $taskArray($task,end)
    } else  {
        if { $new_duration>$taskArray($task,duration) } { 
            set date2 [addweek $taskArray($task,end) [expr $new_duration-$taskArray($task,duration)]]
        } else {
            set date2 $taskArray($task,end)
        }
    }
    set freeweeks $taskArray($task,idle_weeks)
    set freeweeks [concat $freeweeks $taskArray(project,idle_weeks)]
    set $taskArray($task,start) $task_start
    # kydn lpi aikavli date1 date2
    while {[aolderb $date1 $date2]} {
            set x2 [expr $x1+20]
            set y2 [expr $y1+20]
            getweekyear $date1 week1 year2
            # tarkistetaan mahd. rokuliviikko
            if {[lsearch -exact $freeweeks $date1 ]!=-1 && !$early} {
                draw_rectangle $task orange $week1 $year2 0
                incr x1 20 
                set date2 [addweek $date2 1]
                set taskArray($task,end) [addweek $taskArray($task,end) 1] 
                set date1 [addweek $date1 1]
            } else { 
                # tarkistetaan mahd. tehtvn aikaisempi pttyminen
                if {$early} { 
                    draw_rectangle $task white $week1 $year2 0
                    incr x1 20 
                    set date1 [addweek $date1 1]
                } else  {
                    # tarkistetaan tyviikot sek mahdollinen uusikesto
                     while {1} {
                        # alkuperinen kesto
                        if {$completed_weeks<$taskArray($task,duration)} { 
                            draw_rectangle $task dgreen $week1 $year2 0
                            incr completed_weeks 1
                            incr x1 20 
                            break 
                        }
                        # uusi tehtvn kesto
                        if {$completed_weeks<$taskArray($task,new_duration)} { 
                            draw_rectangle $task red $week1 $year2 0
                            incr completed_weeks 1
                            #set date2 [addweek $date2 1]
                            set taskArray($task,end) [addweek $taskArray($task,end) 1]
                            incr x1 20 
                            break 
                        }
                        break    
                     } 
                    set date1 [addweek $date1 1]
                }    
            }
        }
    # talletetaan kunkin tehtvn osalta viimeiseksi ksitelty koordinaatit.
    # Nit tarvitaan kun esitetn liukuma taaksepin (handle_rear)
    set taskArray($task,x1) $x1 
    set taskArray($task,y1) $y1 

    # tarkistetaan tilanne jossa tehtv pttynyt ilman ett on tehty yhtn
    # viikkoa. Tllin mahd. seuraajatehtv voi aloittaa aikaisemmin. 
    if {$early} { 
        if {$completed_weeks==0} { 
            set orig_end $taskArray($task,start)
            set orig_end [addweek $orig_end -1] 
        }
        # seurajatehtv aloittaa tmn tehtvn lopetuksen perusteella joka
        # edell asetettiin 
        set taskArray($task,end) $orig_end
        # jos tehtv pttyi, asetetaan x1-koord. 0. jolloin ko. tehtvn taaksepin
        # liukumia ei esitet (handle_rear)
        set taskArray($task,x1) 0
    }
}
#---------------------------------------------------------------------------
# Funktio make_tasks    
# Parametrit:               taskprop        -taulukko
#                           task            -string
#                           week            -string
#                           task_start      -string
#                           task_end        -string
#                           true_start      -string
#                           prev_end        -string
#                           future          -integer (boolean)
#                
# Ksiteltv tehtv kydn lpi alkaen ensimmisest tark.viikosta
# jatkuen tehtvn viimeiseen viikkoon asti.
#
# Ulkoiset vaikutukset:     Tulostaa kanvaasiin .raami.sproCanv ksiteltvn
#                           tehtvn nimen.
#
# Kutsuu aliohjelmia:       make_to_currentweek
#                           make_to_currentweek
#                           make_to_end
# 
# Palautusparametrit:       -
#
# Historia:
#---------------------------------------------------------------------------
proc make_tasks {taskprop task week task_start task_end true_start prev_end \
    startdate future} { 

    upvar $taskprop taskArray
    global x1 x2 y1 y2 Const
    set x1 10
    set completed_weeks 0    
    set early 0                
    set orig_end 0

    # tulostetaan kanvaasiin ksiteltvn tehtvn nimi    
    .raami.sproCanv create text $x1 [expr $y1+10] -text "$task" \
    -tags T$task -anchor w -font $Const(name_font)
    incr x1 100

    # ksitelln viikot tehtvn alkuun saakka ..
    make_to_taskstart taskArray $task $week task_start task_end $true_start \
        prev_end $startdate $future

    # ja edelleen tehtvn alusta tarkasteluviikkoon saakka ..
    make_to_currentweek taskArray $task $task_start $week $prev_end \
        completed_weeks early orig_end

    # sek lopuksi viikot tarkasteluviikosta tehtvn loppuun saakka
    make_to_end taskArray $task $task_start $week completed_weeks $early \
        $future $orig_end

    #asetetaan tehtvn aluksi sen todellinen alku
    set taskArray($task,start) $true_start
    #siirrytn seuraavalle riville
    incr y1 30     

} 
#------------------------------------------------------------------------------
# Funktio make_task_order    
# Parametrit:               weekyear        -string
#
# Tehtvt jrjestetn suoritukseen edeltjsuhteiden perusteella  
#
# Ulkoiset vaikutukset:     Tulostaa kanvaasin .raami.sproCanv "Tehtvt" -rivin.
#
# Kutsuu aliohjelmia:       read_Resources (read.tcl)
#                           read_Project_name (read.tcl)
#                           read_Main_info (read.tcl)
#                           read_Work_weeks (read.tcl)
#                           read_Free_weeks (read.tcl)
#                           get_startdate
#                           make_end_dates 
#                           check_cycles 
#                           getweekyear (share.tcl)
#                           aolderb
# 
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Poistettu ikkunan
#                                       "pakko-sijoittaminen" kyttmll
#                                       apuna muuttujaa Const(windowresizeable)
#------------------------------------------------------------------------------
proc make_task_order {weekyear} {

    # poistetaan kanvaasista vanhat viikko/resurssi-merkinnt ..    
    .raami.sproCanv addtag testi all 
    .raami.sproCanv delete testi 


    # luetaan tiedostot
    read_Resources resourceArray resources  ;#luetaan resurssitiedot
    read_Project_name taskArray             ;#luetaan projektin nimi ja lopetusviikko
    read_Main_info taskArray tasks resources;#luetaan tehtvien perustiedot 
    read_Work_weeks taskArray tasks         ;#luetaan tehdyt viikot+tot.rokulit 
    read_Free_weeks  taskArray tasks        ;#luetaan suunnitellut rokulit

    set startdate [get_startdate taskArray $tasks]

    make_end_dates taskArray $tasks         ;#luodaan alustava tehtvien lopetusviikko
    check_cycles taskArray $tasks           ;#tarkistetaan mahdollinen syklisyys    

    global x1 x2 y1 y2 Const 
    set x1 10
    set y1 30
    set y2 0
    set x2 0
    set task_number [llength $tasks]
    set task_id 0
    getweekyear $weekyear week year

    if {[aolderb $weekyear $startdate]} {
        set startdate $weekyear
    }

    # tulostetaan kanvaasiin vliotsikko "Tehtvt"    
    set x1 10
    .raami.sproCanv create text $x1 [expr $y1+10] -text "Tehtvt" \
    -anchor w -font $Const(headline_font) 
    incr y1 30
    set task_id 0

    # toistetaan kunnes murtaudutaan ulos
    while {1} {
        # ksitelln kaikki tehtvt. Ksiteltv tehtv valitaan task_id:n perusteella.
        # Ksitelty tehtv merkitn prosessoiduksi.
        while {$task_id<$task_number} { 
            set task [lindex $tasks $task_id] 
            if {$taskArray($task,processed)} {
                incr task_id 1
            } else {
                # tehtv ei aikaisemmin ksitelty. Murtaudutaan silmukasta 
                break 
            }
        }
        # jos kaikki tehtvt jo prosessoitu, voidaa murtaa silmukka "while {1}"
        if {$task_id>=$task_number} { 
            break 
        }
        if {$taskArray($task,forerunner)==""} {
            # tehtvll ei ole edeltji. Tehtv voidaan ksitell heti, vlittmtt
            # mahdollisista edeltjist.
            set task_start $taskArray($task,start)
            set task_end $taskArray($task,end) 
            set null -1/-1    

            if {[aolderb $weekyear $task_start]} {
                make_tasks taskArray $task $weekyear $task_start $task_end $null \
                    $null $startdate 1
            } else { 
                make_tasks taskArray $task $weekyear $task_start $task_end $null \
                    $null $startdate 0
            }
            set taskArray($task,processed) 1
            set task_id 0

        } else { 
            # tehtvll on edeltji. Nyt tytyy tarkistaa ett edeltjt on ksitelty
            # jo aikaisemmin.
            set forerunners $taskArray($task,forerunner)
            set status 1

            # asetetaan oletus alkukohdaksi nykyisen tehtvn suunniteltu aloitus
            set true_start $taskArray($task,start)
            set prev_end -1/-1

            # tarkistetaan ett edeltjt on ksitelty. Jos lydetn tehtv jota ei
            # ole ksitelty, valitaan se seuraavaksi ksiteltvksi. Samalla merkitn
            # edeltj-tehtvn lopetusviikko muistiin.
            foreach forerunner $forerunners {
                if {$taskArray($forerunner,processed)!=1} {
                    set status 0
                    set task_id [lsearch -exact $tasks $forerunner]
                    break 
                } 
                if {[aolderb $prev_end $taskArray($forerunner,end) ]} {
                    set prev_end $taskArray($forerunner,end)
                }
            } 

            # tehtvn edeltjt oli ksitelty ..      
            if {$status} { 
                if {[aolderb $true_start $prev_end ]} { 
                    set true_start [addweek $prev_end 1] 
                } 
                set taskArray($task,processed) 1 
                set task_start $taskArray($task,start) 
                set task_end $taskArray($task,end) 

                # onko tehtvn aloitus tulevaisuudessa ..
                if {[aolderb $weekyear $true_start]} { 
                    make_tasks taskArray $task $weekyear $task_start $task_end \
                        $true_start $prev_end $startdate 1
                } else {
                    make_tasks taskArray $task $weekyear $task_start $task_end \
                        $true_start $prev_end $startdate 0
                }
                set task_id 0
            }
        }
    }
    # talletetaan koordinaattipisteet vliaikaisesti
    set x1_vara $x1
    set x2_vara $x2
    set y1_vara $y1
    set y2_vara $y2

    # tarkistetaan tehtvien liukuma taaksepin.    
    handle_rear taskArray $tasks $weekyear

    # palautetaan alkuperiset koordinaattipisteet
    set x1 $x1_vara 
    set x2 $x2_vara 
    set y1 $y1_vara 
    set y2 $y2_vara 

    set windowname $taskArray(project)


    # ksitelln resurssit
    make_resources resourceArray taskArray $resources $tasks $weekyear $startdate

    # piirretn kanvaasiin pystyakselit, ylriville vuodet+viikot
    draw_markers $resources $tasks taskArray $weekyear $startdate

    # mritelln ikkunan kyttm vieritysalue + ohjelmapalkin sislt
    .raami.sproCanv config -scrollregion "0 0 [expr $x1+29] [expr $y2+20]" -width 500
    wm title . "Spro: $windowname $week/$year"

    # jos ikkunaa ei pivitetty aikaisemmin, asetetaan ikkunan oletuskoko.
    # Myhemmt ikkunan koon muutokset eivt muuta ikkunan sijaintia/kokoa
    # ruudulla kun ohjelma seuraavan kerran pivitetn. 
    if {!$Const(windowresizeable)} {
        wm geometry . +7+25
        wm geometry . [expr $x1+29]x[expr $y2+70]    
        set Const(windowresizeable) 1
    }

}
