




###############################################################################
#       OHJELMA:            SPRO
#       MODULI:             read.tcl
#       TEKIJ:             Anssi Laakso
#       MUUTOSHISTORIA:     5.7.1999        read_Work_weeks
#                                           read_Resources
#                                           read_Free_weeks
#                                           read_Project_name 
#                                           read_Main_info
#
###############################################################################





#------------------------------------------------------------------------------
# Funktio read_Work_weeks    
# Parametrit:               taskprop        -taulukko
#                           tehtavat        -lista
#                
# Luetaan tiedostosta viikot.txt tehtvittin tehdyt viikot. Tiedosto lytyy
# ympristmuuttujan SPRO_ENV osoittamasta hakemistosta. Talletetaan tehdyt
# viikot taulukkoon taskprop. Parametrista tehtavat saadaan selville ne
# tehtavat, joiden tehtavatiedot taytyy lyty.
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       error_message (share.tcl)
#                           dateformat_ok (share.tcl)
# 
# Historia:                 5.7.1999    Poistettu "pakkokommenttirivi" luku 
#                                       tiedoston alusta; poistettu 
#                                       virheilmoitus jos perustiedoissa olevaa 
#                                       tehtv ei esiinny tiedostossa 
#                                       viikot.txt; muutettu ympristmuuttujan
#                                       nimi
#------------------------------------------------------------------------------
proc read_Work_weeks {taskprop tehtavat} {

    upvar $tehtavat tasks
    upvar $taskprop taskArray
    global env
    set weekFile viikot.txt

    if {[catch {set work_file [open $env(SPRO_PROJ)/$weekFile r]}]} {
        error_message $weekFile none 1 
    } 

    set found_tasks {}
    set line_number 0

    while {[gets $work_file line]>=0} { 
        incr line_number 1
        # lytyyk erotinmerkki rivilt
        if [string match *:* $line] {      
            set taskweeks [split $line ":"]
        } else {continue}

        # asetetaan tehtvnimi
        set taskname [lindex $taskweeks 0]

        # lytyyk tehtv perustiedoista ..
        if {[lsearch $tasks $taskname]==-1} {
            error_message $weekFile $line_number 4
        } else  {
            # tai onko luettu jo aikaisemmin
            if {[lsearch -exact $found_tasks $taskname]!=-1} {
                error_message $weekFile $line_number 12
            } else  {
                lappend found_tasks $taskname
            }
        }    
        # asetetaan tehtvviikot
        set taskweeks [lindex $taskweeks 1]
        set dates {}
        # tarkistetaan tehtvviikot
        foreach t $taskweeks {
            if {$t=="X"} {
                if {[lsearch $dates "X"]==-1} {
                    lappend dates $t 
                } else {
                    error_message $weekFile $line_number 16
                }
            } else  {
                if {[dateformat_ok $t]}  {
                    lappend dates [set t [add_century $t]]
                } else  {
                    error_message $weekFile $line_number 2
                }
            }
        }
        set taskArray($taskname,work_weeks) $dates
    }

    # jos luettuja tehtvi ei lydy verrattaessa perustietoihin, asetetaan
    # ko. tehtvien tehdyt viikot tyhjiksi ""
    foreach task $tasks {
        if {[lsearch -exact $found_tasks $task ]==-1} {
            set taskArray($task,work_weeks) ""
        }
    }
    close $work_file
}
#------------------------------------------------------------------------------
# Funktio read_Resources    
# Parametrit:               resprop         -taulukko
#                           res             -lista
#
# Luetaan tiedostosta resper.txt resurssien perustiedot.  Tiedosto lytyy
# ympristmuuttujan SPRO_ENV osoittamasta hakemistosta. Resurssien perustiedot
# talletetaan taulukkoon resprop. Listasta res saadaan selville ne resurssit,
# joiden tietojen on lydyttv ko. tiedostosta.
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       error_message   (share.tcl)
#                           dateformat_ok   (share.tcl)
#                           number_ok       (share.tcl)
#
# Palautusparametrit:       -
# 
# Historia:                 5.7.1999    Poistettu "pakkokommenttirivin" 
#                                       lukeminen; muutettu
#                                       ympristmuuttujan nimi
#------------------------------------------------------------------------------
proc read_Resources {resprop res} {

    upvar $res resources
    upvar $resprop resourceArray
    global env
    set resourceFile resper.txt
    set found_resources {}
    set resources {}    

    if {[catch {set res_file [open $env(SPRO_PROJ)/$resourceFile r]}]} { 
        error_message $resourceFile none 1 
    } 

    set line_number 0    
    while {[gets $res_file line]>=0} { 
        incr line_number 1

        # Luetaan resurssin nimi
        if [string match Nimi:* $line] {
            set resourcename [string range $line 5 [string length $line]]
            if {$resourcename==""} {
                error_message $resourceFile $line_number 5
            }
        } else {continue}

        # Luetaan resurssin mr
        gets $res_file line
        incr line_number 1
        if [string match Mr:* $line] {
            set quantity [string range $line 6 [string length $line]] 
            if {![number_ok $quantity]} {
                error_message $resourceFile $line_number 3
            }
        } else {continue}

        # Luetaan resurssin poikkeusviikko mrt
        gets $res_file line
        incr line_number 1
        if [string match Viikko|mr:* $line] {
            set exception [string range $line 13 [string length $line]] 
            set date {}
            foreach p $exception {
                set d [lindex [split $p |] 0]
                if {![dateformat_ok $d]} {
                    error_message $resourceFile $line_number 2
                }
                set d [add_century $d]
                set qty [lindex [split $p |] 1]
                if {![number_ok $qty]} {
                    error_message $resourceFile $line_number 3
                }
                set p [concat $d|$qty]
                lappend date $p
               }
 
        } else {continue}

        lappend resources $resourcename

        if {[lsearch $found_resources $resourcename]!=-1} {
            error_message $resourceFile $line_number 6
        }
        lappend found_resources $resourcename
        set resourceArray($resourcename,quantity) $quantity
        set resourceArray($resourcename,exception) $date
    }    
    close $res_file
}
#------------------------------------------------------------------------------
# Funktio read_Free_weeks    
# Parametrit:               taskprop        -taulukko
#                           tehtavat        -lista
#
# Luetaan tiedostosta rokulit.txt rokuliviikot. Tiedosto lytyy
# ympristmuuttujan SPRO_ENV osoittamasta hakemistosta. Talletetaan
# rokulitiedot taulukkoon taskprop. Listasta tehtavat saadaan selville
# luettavien tehtvien nimet.
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       error_message(share.tcl) 
#                           dateformat_ok (share.tcl) 
#                           add_century (share.tcl)
#
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Poistettu "pakkokommenttirivin" 
#                                       lukeminen; lisksi poistettu 
#                                       virheilmoitus jos perustiedoista 
#                                       lytyv tehtv ei lydy 
#                                       tiedostosta rokulit.txt; muutettu
#                                       ympristmuuttujan nimi
#
#------------------------------------------------------------------------------
proc read_Free_weeks {taskprop tehtavat} {

    global env
    upvar $tehtavat tasks
    upvar $taskprop taskArray
    set freeFile rokulit.txt
    set taskArray(project,idle_weeks) "" 
    set file_not_found [catch {set idle_file [open $env(SPRO_PROJ)/$freeFile r]}]

    if {$file_not_found} { error_message $freeFile none 1 }

    set found_tasks {}
    set line_number 0
    while {[gets $idle_file line]>=0} { 
        incr line_number 1
        # lytyyk erotinmerkki 
        if [string match *:* $line] {
            set taskrokulit [split $line ":"] ;
        } else { continue }

        # tehtvn nimi ..        
        set taskname [lindex $taskrokulit 0]
        if {[lsearch -exact $tasks $taskname]==-1 } {
            if {$taskArray(project)!=$taskname} {
                error_message $freeFile $line_number 4
            }
        } else  {
            if {[lsearch -exact $found_tasks $taskname]!=-1} {
                error_message $freeFile $line_number 7
            }
        lappend found_tasks $taskname
        }

        # tehtvn rokuliviikot
        set taskrokulit [lindex $taskrokulit 1]
        set dates {}
        foreach t $taskrokulit {
            if {[dateformat_ok $t]} {
                lappend dates [set t [add_century $t]]
            } else  {
                error_message $freeFile $line_number 2        
            }
        }

        if {$taskArray(project)==$taskname} { 
            set taskArray(project,idle_weeks) $dates
        } else {
            set taskArray($taskname,idle_weeks) $dates
        }   

    }
    foreach task $tasks {
            
        if {[lsearch -exact $found_tasks $task ]==-1} {
            set taskArray($task,idle_weeks) ""
        }
    }   
    close $idle_file
}
#------------------------------------------------------------------------------
# Funktio read_Project_name    
# Parametrit:               taskprop        -taulukko
#
# Luetaan tiedostosta projekti.txt projektin nimi SEK
# projektin lopetusviikko. Tiedosto lytyy
# ympristmuuttujan SPRO_ENV osoittamasta hakemistosta. Tiedot talletetaan
# taulukkoon taskprop.
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       error_message (share.tcl)
#                           add_century (share.tcl)
#                           dateformat_ok (share.tcl)
#
# Historia:                 5.7.1999    Poistettu "pakkokommenttirivin" 
#                                       lukeminen; muutettu
#                                       ympristmuuttujan nimi; listty 
#                                       virhetarkistus mahd. kokonaan 
#                                       puuttuvien rivien tapauksessa
#------------------------------------------------------------------------------
proc read_Project_name {taskprop} {

    upvar $taskprop taskArray
    global env
    set projectFile projekti.txt

    if  [catch {set projFile [open $env(SPRO_PROJ)/$projectFile r]}] { 
        error_message $projectFile none 1 }     

    set line_number 0
    set projectname ""
    set date ""
    
    while {[gets $projFile line]>=0} {
        # Luetaan projektin nimi
        incr line_number 1
        if [string match Projekti:* $line] {
            set projectname [string range $line 9 [string length $line]] 
            if {$projectname==""} {
                error_message $projectFile $line_number 9 
            }
        } else {continue}

        # Luetaan projektin loppuviikko
        gets $projFile line
        incr line_number 1
        if [string match Loppuviikko:* $line] {
            set project_end [string range $line 12 [string length $line]]
            if {[dateformat_ok $project_end]} {
                set date [add_century $project_end] 
            } else  {
                error_message $projectFile $line_number 2
            }
        } else {continue}
    }

    # virhetarkistus jos ko. tiedot puuttuvat kokonaan!!!
    if { $projectname=="" || $date==""} {
            error_message $projectFile $projectFile 17 
    }
    
    set taskArray(project) $projectname 
    set taskArray(project_end) $date 

    close $projFile
}
#------------------------------------------------------------------------------
# Funktio read_Main_info   
# Parametrit:               taskprop        -taulukko
#                           tehtavat        -lista
#                           res             -lista
#
# Luetaan tiedostosta perust.txt tehtvien perustiedot. Tiedosto lytyy
# ympristmuuttujan SPRO_ENV osoittamasta hakemistosta. Tehtvien perustiedot
# talletetaan taulukkoon taskprop.
# HUOM! Resurssien perustiedot oltava jo luettu tai virheilmoitus
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       error_message (share.tcl)
#                           add_century (share.tcl)
#                           dateformat_ok (share.tcl)
#                           number_ok (share.tcl)
#
# Palautusparametrit:       -
#
# Historia:                 5.7.1999    Poistettu "pakkokommenttirivin" 
#                                       lukeminen; muutettu
#                                       ympristmuuttujan nimi
#------------------------------------------------------------------------------
proc read_Main_info {taskprop tehtavat res} {

    upvar $taskprop taskArray
    upvar $tehtavat tasks
    upvar $res resources 
    global env
    set mainFile perust.txt
    set tasks {}

    if {[catch {set maininfo [open $env(SPRO_PROJ)/$mainFile r]}]} { 
         error_message $mainFile none 1 
    }
    set line_number 0

    while {[gets $maininfo line]>=0} {
        incr line_number 1
        # Luetaan tehtvn nimi
        if [string match Nimi:* $line] {
            set taskname [string range $line 5 [string length $line]]
            if {$taskname==""} {
                error_message $mainFile $line_number 10 
            }
        } else {continue}

        # Luetaan edeltjt
        gets $maininfo line
        incr line_number 1
        if [string match Edeltajat:* $line] {
            set forerunners [string range $line 10 [string length $line]] 
        } else {continue}

         # Luetaan tehtvn aloitus       
        gets $maininfo line
        incr line_number 1
        if [string match Aloitus:* $line] {
            set aloitus [string range $line 8 [string length $line]]
            if {[dateformat_ok $aloitus]} {
                set date [add_century $aloitus] 
            } else  {
                error_message $mainFile $line_number 2
            }
        } else {continue}

        # Luetaan tehtvn kesto
        gets $maininfo line
        incr line_number 1
        if [string match Kesto:* $line] {
            set duration [string range $line 6 [string length $line]]
            if {![number_ok $duration]} {
                error_message $mainFile $line_number 3 
            }
        } else {continue}

        # Luetaan tehtvn uusikesto
        gets $maininfo line
        incr line_number 1
        if [string match UUSIKESTO:* $line] {
            set new_duration [string range $line 10 [string length $line]] 
            if {$new_duration!=""} {
                if {![number_ok $new_duration]} {
                        error_message $mainFile $line_number 3
                }
            }
        } else {continue}

        # Luetaan tehtvn kyttmt resurssit
        gets $maininfo line
        incr line_number 1
        if [string match Resurssit:* $line] {
            set task_resources [string range $line 10 [string length $line]] 
            foreach p $task_resources { 
                set resource_name [lindex [split $p |] 0]
                set resource_quantity [lindex [split $p |] 1]
                if {[lsearch -exact $resources $resource_name]==-1} {
                    error_message $mainFile $line_number 11
                }
                if {![number_ok $resource_quantity]}  {
                        error_message $mainFile $line_number 3
                }
            }
        } else {continue}

        # Talletetaan luettu tehtv listaan tasks
        if {[lsearch -exact $tasks $taskname]==-1} {
            lappend tasks $taskname
        } else  {
            error_message $mainFile $line_number 12
        }

        # Viedn taulukkoon kunkin tehtvn tiedot
        set taskArray($taskname,forerunner) $forerunners 
        set taskArray($taskname,start) $date 
        set taskArray($taskname,duration) $duration
        set taskArray($taskname,new_duration) $new_duration 
        set taskArray($taskname,resources) $task_resources 
        set taskArray($taskname,processed) 0
    }
    # Tarkistetaan edeltvien tehtvien olemassaolo SEK
    # ettei tehtv sisll linkki itseens
    foreach task $tasks {

        foreach pre $taskArray($task,forerunner) {
            if {[lsearch -exact $tasks $pre]==-1} {
                set b [exec grep Edeltajat:$taskArray($task,forerunner) \
                    $env(SPRO_PROJ)/$mainFile -n] 
                error_message $mainFile $b 13
            }
            if {$pre==$task} { 
                #tarkistetaan ettei sisll linkki itseens
                set b [exec grep Edeltajat:$taskArray($task,forerunner) \
                    $env(SPRO_PROJ)/$mainFile -n] 
                  error_message $mainFile $b 14
            }
        }
    }
    close $maininfo
}
