




###############################################################################
#       OHJELMA:            SPRO
#       MODULI:             share.tcl
#       TEKIJ:             Anssi Laakso
#       MUUTOSHISTORIA:     5.7.1999    print_info
#                                       error_message
#
#
###############################################################################





#------------------------------------------------------------------------------
# Funktio number_ok    
# Parametrit:               number      -integer
#
# Funktio palauttaa totuusarvon 0 tai 1, jos parametrina saatu numero sislt
# laittomia merkkej (aakkosia tms.) tai ei ole arvoalueella (1-n).
#
# Ulkoiset vaikutukset:     -
# Kutsuu aliohjelmia:       -
#                        
# Palautusparametrit:       integer (boolean)
#
# Historia:
#------------------------------------------------------------------------------
proc number_ok {number} {

    if {$number<=0 || [catch {expr 1 + $number}] || \
            [string match *\[*-+,.\]* $number]} {
                    return 0
            }
    return 1
}
#------------------------------------------------------------------------------
# Funktio getweekyear    
# Parametrit:               weekyear        -string (esim 12/1999)
#                           Week            -integer (esim 12)
#                           Year            -integer (esim 1999)    
#
#
# Funktio palauttaa saamastaan parametrista weekyear sek viikon ja vuoden.
#
# Ulkoiset vaikutukset:     -
# Kutsuu aliohjelmia: 
#                        
# Palautusparametrit:       year            -integer 
#                           week            -integer
#
# Historia:
#------------------------------------------------------------------------------
proc getweekyear {weekyear Week Year} {

    upvar $Week week
    upvar $Year year

    set week [split $weekyear "/"] 
    set year [lindex $week 1]
    set week [lindex $week 0]
}
#------------------------------------------------------------------------------
# Funktio add_century
# Parametrit:               weekyear        -string (esim. 12/99)
#
# Funktio lis saamansa parametrin weekyear alkuun vuosisadan. Jos weekyear 
# vuosi on vlilt 90-99, palauttaa funktio pivyksen jossa vuosi on 1990-1999.
# Jos weekyear vuosi on vlilt 00-89, palauttaa funktio pivyksen jossa vuosi 
# on 2000-2089.
#
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       getweekyear
#
# Palautusparametrit:       weekyear        -string (esim 12/1999)
#
# Historia:
#------------------------------------------------------------------------------
proc add_century {weekyear} {

    getweekyear $weekyear taskweek taskyear

    # mritelln vuosisata vuoden 90 perusteella 
    if {$taskyear>=90} {
        set taskyear [concat 19$taskyear]
    } else {
        set taskyear [concat 20$taskyear]
    }
    return [concat $taskweek/$taskyear]
}
#---------------------------------------------------------------------
# Funktio dateformat_ok    
# Parametrit:               date
#
# Tarkistetaan pivyksen date oikea muoto. Oltava muotoa esim "11/99"
# 
# Ulkoiset vaikutukset:     -
#
# Kutsuu aliohjelmia:       - 
#
# Palautusparametrit:       integer (boolean)
#          
# Historia:              
#---------------------------------------------------------------------
proc dateformat_ok {date} {

    getweekyear $date week_date year_date 
    set week_length [string length $week_date] 
    set year_length [string length $year_date]

    # pakotetaan virhe, jos viikossa tai vuodessa laittomia merkkej
    if {[catch {expr $year_date + $week_date}]} { 
        return 0
    }

    # merkkijonon pituus oikea eik erikoismerkkej -. tms. viikkomerkinnss
    if  {$week_length<1||$week_length>2||[string match *\[-+.*\]* $week_date]} {
        return 0
    }

    # merkkijonon pituus oikea vuosimerkinnss
    if {$year_length!=2} { 
        return 0
    }

    # viikko oikealla vlill 1-53
    if {$week_date>53||$week_date<1} {
        return 0
    }

    # ei negatiivista arvoa eik erikoismerkkej vuosimerkinnss
    if {$year_date<0||[string match *\[.-+\]* $year_date]}  {
        return 0
    }
    return 1
}
#------------------------------------------------------------------------------
# Funktio error_message    
# Parametrit:               type            -string
#                           message         -string
#                           code            -integer   
#                                                       
# Funktio tulostaa viesti-ikkunaan otsikon type ja sislln message.
# Tulostettava virheilmoitus valitaan parametrin code perusteella.
#
# Ulkoiset vaikutukset: Tulostaa viesti-ikkunan nytlle
#
# Kutsuu aliohjelmia:       - 
# 
# Palautusparametrit:       -
#
# HUOM!    Lopettaa ohjelman suorituksen aina exit-koodilla !
#
# Historia:                 5.7.1999    Poistettu tarpeettomia virheilmoituksia
#------------------------------------------------------------------------------
proc error_message {infoA infoB code} {
    
    # Oletusarvoisesti viesti-ikkunan otsikko on seuraava, ellei sit
    # aseteta myhemmin uudelleen
    set type "Tiedoston $infoA lukuvirhe"
    switch $code {

            1    {
                set type "Tiedoston lukuvirhe"
                set message "Tiedostoa \"$infoA\" ei lydy! \
                \nTarkista tiedostojen olemassaolo."
            }
            2    {
                set message "Rivi $infoB:\
                    Rivill virheellinen pivys."
            }
            3    {
                set message "Rivi $infoB: Virheellinen numeroarvo"
            }
            4     {
                set message "Rivi $infoB: Tehtv puuttuu perustiedoista."
            }
            5    {
                set message "Rivi $infoB: Resurssin nimi puuttuu."
            }
            6     {
                set message "Rivi $infoB Resurssi esiintyy kahteen kertaan."
            }
            7    {
                set message "Rivi $infoB: Tehtv esiintyy kahteen kertaan."
            }
            9    {
                set message "Rivi $infoB: Projektin nimi puuttuu."
            }
            10    {
                set message "Rivi $infoB: Tehtvn nimi puuttuu."
            }
            11    {
                set message "Rivi $infoB: Resurssia ei olemassa"
            }
            12     {
                set message "Rivi $infoB: Tehtv esiintyy kahteen kertaan."
            }
            13    {
                set message "Rivit:\n$infoB. Edeltj ei lydy."
            }
            14    {
                set message "Rivi:\n$infoB. Tehtvn edeltj ei voi olla \
                tehtv itse"
            }
            16     {
                set message "Rivi $infoB: Lopetusmerkki esiintyy kahteen kertaan."
            }
            17  {
                set message "Projektin nimi ja/tai lopetusviikko puuttuu."
            }
            18    {
                set type "Tehtvt muodostavat syklin" 
                set message "Rivi(t):\n$infoB"
            }
    }
    tk_messageBox -icon error -title $type -message $message
    exit 
}
#------------------------------------------------------------------------------
# Funktio print_info  
# Parametrit:               - 
#
# Funktio tulostaa omaan ikkunaan lyhyen selvityksen ohjelman kyttmist
# merkinnist
#
# Ulkoiset vaikutukset:     Tulostaa nytlle info-ikkunan
# Kutsuu aliohjelmia:       -
#                        
# Palautusparametrit:       -   
#
# Historia:                 5.7.1999    Copyright (c) muutettu. Muutettu lisksi
#                                       "liukumaviikko" --> "liukumavara"
#------------------------------------------------------------------------------

proc print_info {} {

    global Const
    
    toplevel    .info
    frame       .info.infoFrame
    pack        .info.infoFrame
    
    set x_point 22
    set y_point 10

    # ikkunan mrittelyj
    canvas      .info.infoFrame.infoCanv -width 500 -height 300 -background white
    pack        .info.infoFrame.infoCanv -expand 1 -fill both -padx 1 -pady 1
    button      .info.lopeta -text OK -command ".buttons.info configure \
                -state active;destroy .info"
    pack        .info.lopeta
    wm geometry .info +100+300
    wm maxsize  .info 500 350
    wm minsize  .info 500 350

    .info.infoFrame.infoCanv create text $x_point $y_point \
    -text "Spro versio $Const(Spro_version)" -anchor w \
    -font *-times-medium-r-normal--16-100-*

    set weekColors [subst {$Const(work_week_spare) $Const(work_week_estimate)\
    $Const(work_week_free) $Const(work_week_premature)\
    $Const(work_week_late)\
    $Const(work_week_transition) $Const(work_week_done)}]

    # alustetaan taulukko halutuilla tehtvien kuvauksilla
    set weekMessage(0)  "Liukumavara, tulevaisuutta "
    set weekMessage(1)  "Arvioitu tyviikko, tulevaisuutta"        
    set weekMessage(2)  "Rokuliviikko, menneisyytt/tulevaisuutta"        
    set weekMessage(3)  "Alitusviikko, menneisyytt/tulevaisuutta"
    set weekMessage(4)  "Ylityviikko, menneisyytt/tulevaisuutta"
    set weekMessage(5)  "Siirtymviikko, menneisyytt/tulevaisuutta"
    set weekMessage(6)  "Tehty viikko, menneisyytt"

    set resourceColors [subst {$Const(resource_usage_20) $Const(resource_usage_50)\
    $Const(resource_usage_80) $Const(resource_usage_120) $Const(resource_usage_MAX)}]

    # lustetaan taulukko halutuilla resurssien kuvauksilla
    set resourceMessage(0) "Resurssin kyttaste 0-20 %"
    set resourceMessage(1) "Resurssin kyttaste 21-50 %"
    set resourceMessage(2) "Resurssin kyttaste 51-80 %"
    set resourceMessage(3) "Resurssin kyttaste 81-120 %"
    set resourceMessage(4) "Resurssin kyttaste >120%"

    .info.infoFrame.infoCanv create text $x_point [expr $y_point+20] -text "Tehtvmerkinnt" \
    -anchor w
    incr y_point 40

    # tulostetaan tehtvmerkinnt silmukassa
    set length [llength $weekColors]
    for {set i 0} {$i<$length} {incr i} {
        .info.infoFrame.infoCanv create rect $x_point [expr $y_point+4] [expr $x_point+10] \
         [expr $y_point-4] -fill [lindex $weekColors $i]
        .info.infoFrame.infoCanv create text [expr $x_point+20] $y_point -text \
        $weekMessage($i) -anchor w
        incr y_point 20
    }

    set y_point 10
    set x_point 300

    .info.infoFrame.infoCanv create text $x_point [expr $y_point+20] -text "Resurssimerkinnt" \
    -anchor w
    incr y_point 40

    # tulostetaan resurssimerkinnt silmukassa
    set length [llength $resourceColors]
    for {set i 0} {$i<$length} {incr i} {
        .info.infoFrame.infoCanv create rect $x_point [expr $y_point+4] [expr $x_point+10] \
         [expr $y_point-4] -fill [lindex $resourceColors $i]
        .info.infoFrame.infoCanv create text [expr $x_point+20] $y_point -text \
        $resourceMessage($i) -anchor w
        incr y_point 20
    }

    set x_point 150
    set y_point 200

    # copyright ..
    .info.infoFrame.infoCanv create text $x_point [expr $y_point+20] -text \
    "\nCopyright (c) 1999 Joensuun Yliopisto, \n\t\tTietojenksittelytieteen \
    laitos \n\n Idea ja suunnittelu: Jorma Sajaniemi\n Toteutus: Anssi Laakso" -anchor w
}
